#
# CMLC.R (Constrained Multilayer Centrality)
#
# Implementation of eigenvector centrality for constrained multilayer networks.
#
# Author: rob.frost@dartmouth.edu
#

#
# Implementation of an interleaved power iteration over an interdependent system of k
# pseudo-eigenvalue problems. Each problem takes the form:
#
# X_i c(i,A) = \lambda v_i
#
# where the function c(i, A) = sum_{j=1}^k a_{i,j} v_j.
# The principal pseudo-eigenvector/eigenvalues for
# this system of interdependent problems can be solved via an interleaved power iteration 
# algorithm where psudo-eigenvalue problems are updated in parallel during each iteration using the
# appropriate pseudo-eigenvector solutions from the prior step.
#
# Inputs:
# 
#   X: List of k (k >= 2) p-by-p Hermitian matrices 
#      for which the principal eigenvectors/eigenvalues will be computed.
#   v1.init: Initial value of principal eigenvector used for all k eigen-problems.
#      If not specified, will default to rep(1/sqrt(p), p).
#   A: A k-by-k matrix whose values capture inter-layer dependencies between
#      the k eigenproblems. Although the elements of A can be positive or negative, each 
#      row must sum to 1. Note that negative weights can lead to negative centrality values.
#      A equals the identity matrix when no dependencies exist.
#   max.iter: The maximum number of iterations.
#   lambda.diff.threshold: If the absolute change in the mean estimated eigenvalue relative to
#       the mean eigenvalue is less than this threshold between iterations, the algorithm will exit.
#   per.layer.convergence: If true, the algorithm only exits if the absolute change in estimated eigenvalues
#       is less than lambda.diff.threshold for all layers.
#   trace: If true, debugging messages will be output.
#   
# Outputs: List with the following elements:
#
#   V1: A p-by-k matrix whose columns contain the principal eigenvectors of the k eigen-problems.
#   lambdas: A length k vector of principal eigenvalues.
#   num.iter: Number of iterations completed.
#   converged: True if convergence was hit.  
#
constrainedMultiplePowerIteration = function(X, 
                                             v1.init, 
                                             A,
                                             max.iter=100, 
                                             lambda.diff.threshold=1e-6, 
                                             per.layer.convergence=FALSE,
                                             trace=FALSE) {
  
  if (!is.list(X)) {
    stop("X must be a list of Hermitian matrices!")
  }
  k = length(X)
  if (k < 2) {
    stop("At least two matrices must be specified!")
  }
  p = 0
  for (i in 1:k) {
    X.i = X[[i]]
    if (nrow(X.i) != ncol(X.i)) {
      stop("Matrix ", i, " is not square!")
    }  
    if (nrow(X.i) < 2) {
      stop("Matrix ", i, " has dimensions less than 2x2!")
    }
    if (i == 1) {
      p = nrow(X.i)
    } else if (nrow(X.i) != p) {
        stop("Matrix ", i, " does not have the same dimensions as the other matrices!")
    }
  }
  if (!is.matrix(A)) {
    stop("A must a matrix!")
  }
  if (nrow(A) != ncol(A)) {
    stop("A must a square matrix!")
  }
  if (nrow(A) != k) {
    stop("Number of rows of A must be ", k) 
  }
  
  # Check that rows sum to 1
  row.sums = rowSums(A)
  if (length(which(row.sums != 1)) > 0) {
    stop("The rows of A must each sum to 1!")
  }
  
  # If some weights are negative, warn user
  if (length(which(A < 0)) >0) {
    warning("Some dependency weights are negative! Be aware that this can result in negative constrained centrality values and/or slow algorithm convergence.")
  }
  
  # If not specified, initialize principal eigenvector to unit vector with equal values
  if (missing(v1.init)) {
    v1.init = rep(1/sqrt(p), p)        
  } 
  
  # Create the matrix that will hold the updated principal eigenvectors
  V1 = matrix(rep(v1.init,k), nrow=p, ncol=k, byrow=F)
  
  # Initialize principal eigenvalues
  lambdas = rep(NA, k)
  lambda.diffs = list()
  
  # Update using modified power iteration
  converged = FALSE
  for (i in 1:max.iter) {
    
    if (trace) {
      message("Iteration ", i)
    }
    
    #lambda.diffs[[i]] = 0
    
    lambdas.est = rep(0,k)
    for (j in 1:k) {      
      # Update principal eigenvector
      X.layer = X[[j]]
      v1 = X.layer %*% (V1 %*% A[j,])
      # Renormalized to unit length
      v1 = as.vector(v1/sqrt(sum(v1^2)))
      # Update matrix
      V1[,j] = v1    
      # Compute principal eigenvalue
      lambdas.est[j] = t(v1) %*% X.layer %*% (V1 %*% A[j,]) 
    }
    
    if (trace) {
      message(paste("Updated lambdas:", paste(lambdas.est, collapse=", ")))
    }    
    
    # Check if the difference is less than the threshold, exit
    if (i > 1) {
      rel.diffs = abs(lambdas.est - lambdas)/lambdas.est
      mean.rel.diff = mean(rel.diffs)
      #lambda.diffs[[i-1]] = mean.rel.diff
      rel.diff.to.test = mean.rel.diff
      if (per.layer.convergence) {
        rel.diff.to.test = max(rel.diffs)
      }
      if (rel.diff.to.test <= lambda.diff.threshold){
        if (trace) {
          message("Relative lambda diff less than threshold at iteration ", i, ", exiting.")
        }
        converged=TRUE
        break
      } 
    }    
    lambdas=lambdas.est
  }    
  
  results = list()
  results$V1 = V1
  results$lambdas = lambdas
  results$num.iter = i
  results$converged = converged
  #results$lambda.diffs = lambda.diffs
  
  return (results)
}

#
# Generates an adjacency matrix for a single network that is formed 
# by connecting together each layer in a multilayer network. Specifically,
# if the off-diagonal entry for row i and column j in the inter-layer dependency matrix A is
# not zero, then a directed edge is added with the weight of that entry between all nodes in
# layer i and the same nodes in layer j.
#
# Inputs:
#  -X: A length k list whose entries are the n-by-n adjacency matrices for each layer in a multilayer network with
#       k layers that each represent a network over n nodes.
#  -A: The k-by-k inter-layer dependency matrix. The off-diagonal elements capture the dependency between each pair of layers, which
#      realized by adding inter-layer edges. The diagonal elements are ignored.
#
# Output: The kp-by-kp adjacency matrix for the network created by combining the networks for each of the k layers using inter-layer edges.
#
createMergedAdjacencyMatrix = function(X, A) {
  k = length(X)
  n = nrow(X[[1]])
  #M = Matrix(0, nrow=k*n, ncol=k*n)
  M = matrix(0, nrow=k*n, ncol=k*n)
  # set diagonals in A to 0
  diag(A) = 0
  for (i in 1:k) {
    G = X[[i]] 
    start = 1+(i-1)*n
    end = start+(n-1)
    # update the diagonal block to hold the 
    M[start:end,start:end] = as.matrix(G)
  }
  # update the off-diagonals based on inter-layer dependencies
  # use the Kronecker product of A and the n-by-n identity matrix
  M = M + kronecker(A,diag(rep(1,n)))
  return (M)
}

#
# Computes the eigenvector centrality values for a multilayer network by first generating the adjacency matrix for a
# single merged network using the createMergedAdjacencyMatrix() method and then computing standard eigenvector centrality for this
# merged network. The computed centrality values for each layer are normalized to length 1.
#
# Inputs:
#  X: A length k list whose entries are the n-by-n adjacency matrices for each layer in a multilayer network with
#       k layers that each represent a network over n nodes.
#  A: The k-by-k inter-layer dependency matrix. The off-diagonal elements capture the dependency between each pair of layers, which
#      realized by adding inter-layer edges. The diagonal elements are ignored.
#  normalize.per.layer: If true, the centrality values for each layer are normalized to be unit length. 
#      If all values are 0, which can occur when the merged network is not irreducible, no normalization is performed.
#
# Output: A k-by-p matrix whose columns hold the eigenvector centrality values for each of the k p node networks.
#
interlayerEdgeCentrality = function(X, A, normalize.per.layer=TRUE,
                                    max.iter=100,
                                    lambda.diff.threshold=1e-6) {
  k = length(X)
  n = nrow(X[[1]])
  V = matrix(0, nrow=n, ncol=k)
  if (isDiagonal(A)) { # independent layers so process each separately
    for (i in 1:k) {
      V[,i] = abs(powerIteration(X=as.matrix(X[[i]]),max.iter=max.iter,lambda.diff.threshold=lambda.diff.threshold)$v1)
    }
  } else { # dependent so process merged adjacency matrix
    M = createMergedAdjacencyMatrix(X=X,A=A)
    ec.vals = abs(powerIteration(X=M,max.iter=max.iter,lambda.diff.threshold=lambda.diff.threshold)$v1)
    #ec.vals = abs(eigen(M)$vectors[,1])
    for (i in 1:k) {
      start = 1+(i-1)*n
      end = start+(n-1)
      ec.i = ec.vals[start:end]
      if (normalize.per.layer) {
        # compute length
        len = sqrt(sum(ec.i^2))
        if (len != 0) {
          # normalized for unit length
          ec.i = ec.i/len        
        }
      }
      V[,i] = ec.i
    }
  }
  return (V)
}


#
# Implementation of power iteration to compute the first eigenvector/eigenvalue.
# Includes support for truncating to k non-zero entries on each iteration, i.e., 
# the TPower sparse PCA method of Yuan & Zhang (JMLR, 2013).
#
# Inputs:
# 
#   X: Matrix for which the principal eigenvector/value will be computed.
#   k: Optional truncation parameter. Must be between 1 and p. If specified,
#      the estimated eigenvector will have all loadings except for those with the k
#      largest absolute values set to 0 each iteration.
#   v1.init: Initial value of PC loadings. If not specified, will be initialized to
#      rep(1/sqrt(p), p).
#   max.iter: The maximum number of iterations
#   lambda.diff.threshold: If the change in the estimated eigenvalue is less than this threshold
#      between iterations, the algorithm will exit.
#   trace: If true, debugging messages will be output.
#   
# Outputs: List with the following elements:
#
#   v1: Estimated principal eigenvector.
#   lambda: Estimated principal eigenvalue.
#   num.iter: Number of iterations completed.
#
powerIteration = function(X, k, v1.init, max.iter=10, lambda.diff.threshold=1e-6, trace=FALSE) {
  
  p = ncol(X)
  
  # If not specified, initialize principal eigenvector to unit vector with equal values
  if (missing(v1.init)) {
    v1 = rep(1/sqrt(p), p)        
  } else {
    v1 = v1.init
  }
  
  if (missing(k)) {
    k = p
  } else if (k < 1 | k > p) {
    stop("If specified, k must be between 1 and ", p)
  }
  
  # Initialize principal eigenvalue
  lambda1 = NA
  
  # Update using modified power iteration
  for (i in 1:max.iter) {
    
    if (trace) {
      message("Iteration ", i)
    }
    
    # Standard power iteration update of v1 and lambda1
    v1 = X %*% v1
    v1 = as.vector(v1/sqrt(sum(v1^2)))
    
    if (k != p) {
      # Find k loadings with largest absolute values; set all others to 0
      abs.v1 = abs(v1)
      vars.to.truncate = order(abs.v1, decreasing=F)[1:(p-k)]     
      v1[vars.to.truncate] = 0
      
      # Renormalize the eigenvector
      v1 = as.vector(v1/sqrt(sum(v1^2)))
    }
    
    # Compute principal eigenvalue
    lambda1.est = t(v1) %*% X %*% v1
    
    #    if (trace) {
    #      message("Updated v1 from power iteration: ", paste(v1, collapse=", "))
    #      message("Updated lambda1 from power iteration: ", lambda1.est)      
    #    }    
    
    # Check if the difference is less than the threshold, exit
    if (i > 1) {
      if (abs(lambda1.est - lambda1)/lambda1.est <= lambda.diff.threshold){
        if (trace) {
          message("Lambda1 diff less than threshold at iteration ", i, ", exiting.")
        }
        break
      }
    }    
    lambda1=lambda1.est
  }    
  
  results = list()
  results$v1 = as.vector(v1)
  results$lambda1 = lambda1
  results$num.iter = i
  
  return (results)
}

