\name{constrainedMultiplePowerIteration}
\alias{constrainedMultiplePowerIteration}
\title{Constrained multiple power iteration}
\description{
Implementation of an interleaved power iteration algorithm that
computes principal eigenvectors for a system of k interdependent eigenvalue and pseudo-eigenvalue problems.
Each problem takes the form X_i c(i,A) = lambda v_i, where the function c(i, A) = sum_{j=1}^k a_{i,j} v_j. 
The principal pseudo-eigenvector/eigenvalues for
this system of interdepent problems can be solved via an interleaved power iteration 
algorithm where eigenvalue and pseudo-eigenvalue problems are updated in parallel during each iteration using the
appropriate eigenvector solutions from the prior step. See associated arXiv paper for algorithm full details and
a discussion of how this algorithm can be used to compute eigenvector centrality values for interdependent multilayer networks.
}
\usage{
    constrainedMultiplePowerIteration(X, v1.init, A, max.iter=100, 
                                      lambda.diff.threshold=1e-6, 
                                      per.layer.convergence=FALSE,
                                      trace=FALSE)
}
\arguments{
\item{X}{List of k (k >= 2) p-by-p Hermitian matrices (can be sparse)
for which the principal eigenvectors/eigenvalues will be computed. For analysis of a multilayer network,
these should be the adjacency matrices associated with each network layer. Note that the method does not check that
the layer-specific networks are irreducible (i.e., strongly connected).}
\item{v1.init}{nitial value of principal eigenvector used for all k eigen-problems. 
If not specified, will default to rep(1/sqrt(p), p).}
\item{A}{A k-by-k matrix whose values capture the dependencies 
between the k Hermitian matrices represented by \code{X}. 
Although the elements of \code{A} can be positive or negative, each 
row must sum to 1. Note that the use of negative weights can lead to negative centrality values.
\code{A} equals the identity matrix when no dependencies exist.
For analysis of a multilayer network, these should represent how the importance of adjacent nodes is
computed. See associated arXiv paper for full details.}
\item{max.iter}{The maximum number of iterations of the interleaved power iteration algorithm.}
\item{lambda.diff.threshold}{If the absolute change in the mean estimated eigenvalue relative to
the mean eigenvalue is less than this threshold between iterations, the algorithm will exit.}
\item{per.layer.convergence}{If true, the algorithm only exits if the absolute change in estimated eigenvalues
is less than lambda.diff.threshold for all layers.}
\item{trace}{If true, debugging messages will be output.}
}
\value{
A \code{list} with the following elements:
    \itemize{
     \item "V1": A p-by-k matrix whose columns contain the principal eigenvectors of the k eigen-problems.
     \item "lambdas": A length k vector of principal eigenvalues.
     \item "num.iter": Number of iterations completed.
     \item "converged": True if the algorithm converged. False if \code{max.iter} was hit.
     }
}
\keyword{file}
\examples{
    # Define the adjacency matrices for a 2 layer, 3 node multilayer network
    # In the first layer, node 1 is connected to nodes 2 and 3 with weight 1
    # and nodes 2 and 3 are connected with weight 0.5
    X1 = matrix(c(0,1,1,
                  1,0,0.5,
                  1,0.5,0),
            nrow=3, byrow=TRUE)
    # In the second layer, nodes 1 and 3 are connected to node 2 with 
    # weight 1 and 1 and 3 are connected with weight 0.5
    X2 = matrix(c(0,1,0.5,
                  1,0,1,
                  0.5,1,0),
            nrow=3, byrow=TRUE)
    
    # Compute the eigenvector centrality values of each layer separately
    # Node 1 will have the largest for layer 1 with nodes 2 and 3 equal
    # For layer 2, node 2 will have the largest with nodes 1 and 3 equalå 
    abs(eigen(X1)$vectors[,1])
    abs(eigen(X2)$vectors[,1])
    
    # Compute the associted eigenvalues
    eigen(X1)$values[1]
    eigen(X2)$values[1]
    
    # Combine both adjacency matrices into the multilayer list
    X = list(X1, X2)
    
    # Confirm that the constrained multilayer centrality are approximately
    # equal to the independent centrality values when there are no 
    # interlayer dependencies
    constrainedMultiplePowerIteration(X=X, A=diag(c(1,1)))
    
    # Define a depenency matrix where layer 1 is dependent on layer 2
    A = matrix(c(0,1,
                 0,1),
            nrow=2, byrow=TRUE)
            
    # In this case, the constrained multilayer centrality value for layer 2
    # should equal the independent case. 
    constrainedMultiplePowerIteration(X=X, A=A)
    
    # Define a dependency matrix where layer 1 has a negative dependency on layer 2
    # and a positive dependency on itself. 
    A = matrix(c(1.1,-0.1,
                 0,1),
            nrow=2, byrow=TRUE)
            
    # In this case, the constrained multilayer centrality values for layer 1 will be
    # much for layer 2
    # should equal the independent case. 
    constrainedMultiplePowerIteration(X=X, A=A)
    
    # Enforce convergence separately for each layer
    constrainedMultiplePowerIteration(X=X, A=A, per.layer.convergence=TRUE)    
            
}
