\name{interlayerEdgeCentrality}
\alias{interlayerEdgeCentrality}
\title{Eigenvector centrality for multilayer networks with inter-layer edges}
\description{
Computes eigenvector centrality values for a multilayer network by first generating the adjacency matrix for a
single merged network using the \code{\link{createMergedAdjacencyMatrix}} method and then computing standard eigenvector centrality for this
merged network. If \code{normal.per.layer} is true, the computed centrality values for each layer are normalized to length 1.
Note that this method does not check that the merged network is irreducible (i.e., strongly connected) so the computed centrality values may not
follow normal expectations (e.g., some may be 0).
}
\usage{
    interlayerEdgeCentrality(X, A, normalize.per.layer=TRUE, max.iter=100, lambda.diff.threshold=1e-6)
}
\arguments{
\item{X}{List of k (k >= 2) p-by-p adjacency matrices for a multilayer network.}
\item{A}{A k-by-k matrix whose values capture the dependencies 
between the k network layers represented by \code{X}. 
The off-diagonal elements capture the dependency between each pair of layers, which is
realized by adding directed inter-layer edges. The diagonal elements are ignored.}
\item{normalize.per.layer}{If true, the centrality values for each layer are normalized to be unit length.}
\item{max.iter}{The maximum number of iterations of the power iteration algorithm.}
\item{lambda.diff.threshold}{If the absolute change in the mean estimated eigenvalue relative to
the mean eigenvalue is less than this threshold between iterations, the algorithm will exit.}
}
\value{
A p-by-k matrix whose columns contain the centrality values for each of the k network layers.
}
\keyword{file}
\examples{
    # Define the adjacency matrices for a 2 layer, 3 node multilayer network
    # In the first layer, node 1 is connected to nodes 2 and 3 with weight 1
    # and nodes 2 and 3 are connected with weight 0.5
    X1 = matrix(c(0,1,1,
                  1,0,0.5,
                  1,0.5,0),
            nrow=3, byrow=TRUE)
    # In the second layer, nodes 1 and 3 are connected to node 2 with 
    # weight 1 and 1 and 3 are connected with weight 0.5
    X2 = matrix(c(0,1,0.5,
                  1,0,1,
                  0.5,1,0),
            nrow=3, byrow=TRUE)
            
    # Compute the eigenvector centrality values of each layer separately
    # Node 1 will have the largest for layer 1 with nodes 2 and 3 equal
    # For layer 2, node 2 will have the largest with nodes 1 and 3 equal 
    abs(eigen(X1)$vectors[,1])
    abs(eigen(X2)$vectors[,1])
    
    # Combine both adjacency matrices into the multilayer list
    X = list(X1, X2)
    
    # Compute without inter-layer edges. Since the merged network in this case
    # is not irreducible, only the centrality values for the first layer are non-zero
    # and match the values for the layer-specific analysis.
    interlayerEdgeCentrality(X=X, A=diag(c(1,1)))
    
    # Define a dependency matrix where layer 1 and layer 2 nodes are connected with weight 1
    # undirected edge
    A = matrix(c(0,1,
                 1,0),
            nrow=2, byrow=TRUE)
            
    # The merged network is irreducible, so all nodes will have positive centrality values.
    # These won't match the independent case
    interlayerEdgeCentrality(X=X, A=A)
    
    # Disable per-layer normalization
    interlayerEdgeCentrality(X=X, A=A, normalize.per.layer=FALSE)
}
